class Caller:
    """A class used to mock the `.call` able objects returned from Web3 calls"""
    def call(o=None):
        pass

class Transactor:
    """A class used to mock the `.transact` able objects returned from Web3 transaction calls"""
    def transact(o):
        pass

